#include "ExpaRNAEnsemble.h"
#include <dirent.h>
#include"ExpaRNAOutput.h"

extern "C"
{
	#include <ViennaRNA/fold_vars.h>
	float  fold(const char *sequence, char *structure);
}

void ExpaRNAEnsemble::clearEnsemble()
{
   ensembleMolecules.clear();
   ensemblePatterns = PatternPairMap();
}

void ExpaRNAEnsemble::load_input_data(const vector<string>& my_input)
{
	clearEnsemble();
	Molecule myMol1,myMol2;

	cout << "###############################################################################"<<endl;
	cout << "    ExpaRNA - Exact Pattern Alignment of RNA  - version " << ensOptions.VERSION_ExpaRNA << endl;
	cout << "           (c) copyright by Steffen Heyne - 2008-2009"<<endl;
	cout << "###############################################################################"<<endl<<endl;
	cout << " --> Creating virtual molecules with provided data...";

   readFASTA(my_input[0],myMol1,myMol2);

   cout << "finished!" << endl << endl;
   if (ensOptions.verboseOut) myMol1.printInfo(false);
   if (ensOptions.verboseOut) myMol2.printInfo(false);
   ensOptions.mol1Id = myMol1.getId();
   ensOptions.mol2Id = myMol2.getId();
   ensembleMolecules.insert(make_pair(myMol1.getId(),myMol1));
   ensembleMolecules.insert(make_pair(myMol2.getId(),myMol2));
}

//return string, start from pos start, up to 'break' character, remove all escape chars
std::string getFASTALine(std::string myStr,int start,char myBreak)
{
	int tmppos=start;
	string retStr;
	retStr.clear();
	while ((tmppos<myStr.size()) && isprint(myStr[tmppos]))
	{
		if (myStr[tmppos] == myBreak) break;
		retStr.push_back(myStr[tmppos]);
		++tmppos;
	}
	return retStr;
}

// read input file in FASTA style, ATTENTION: seq/str only in one line each!!!
void ExpaRNAEnsemble::readFASTA(string myFASTAfile, Molecule& myMol1, Molecule& myMol2)
{
	std::ifstream fa_file(myFASTAfile.c_str(),ios::in);

	string line,seq1,seq1_name,seq1_str,seq1_strname,seq2,seq2_name,seq2_str,seq2_strname;

	if (fa_file)
	{
		getline(fa_file,line);

		if (line[0]=='>')
		{
			seq1_name = getFASTALine(line,1,' ');
		}
		else
		{
			cerr <<" Wrong fasta format!" << endl; exit(1);
		}

		getline(fa_file,line);

		seq1 = getFASTALine(line,0,' ');

		getline(fa_file,line);
		if (line[0] != '>')
		{
			seq1_str = getFASTALine(line,0,' ');
			getline(fa_file,line);
			if (line[0]=='>')
			{
				seq2_name = getFASTALine(line,1,' ');
			}
			else
			{
				cerr << seq1_str << line << " wrong fasta format!" << endl; exit(1);
			}
		}
		else
		{
			if (line[0] =='>')
			{
				seq2_name = getFASTALine(line,1,' ');
			}
			else
			{
				cerr <<" wrong fasta format!" << endl; exit(1);
			}
		}
		fa_file >> line;
		seq2 = getFASTALine(line,0,' ');
		line="";
		if (!fa_file.eof())
		{
			fa_file >> line;
			seq2_str = getFASTALine(line,0,' ');
		}
		fa_file.close();

		// check which structure is not present and fold with vienna RNA package
		dangles = 2; // vienna package variable for dangling ends, 2 standard for partition function folding

		if (seq1_str.length()==0)
		{
			int len1=seq1.length();
		    char *c_str1 = new char[len1+1];
		    float my = fold(seq1.c_str(), c_str1);
		    seq1_str = (string)c_str1;
		    seq1_strname = seq1_name + "_mfe";
		}
		else
			seq1_strname=seq1_name+"_str";

		if (seq2_str.length()==0)
		{
			int len2=seq2.length();
			char *c_str2 = new char[len2+1];
			float my = fold(seq2.c_str(), c_str2);
			seq2_str = (string)c_str2;
			seq2_strname=seq2_name+"_mfe";
		}
		else
			seq2_strname=seq2_name+"_str";

		myMol1 = Molecule(seq1_name,seq1,seq1_strname,seq1_str);
		myMol2 = Molecule(seq2_name,seq2,seq2_strname,seq2_str);
	}
	else
    {
       cerr << "Could not open File:" << myFASTAfile << endl;
       exit(1);
    }
	fa_file.close();
}


// update the molecules and the ensemblePatterns with new Patterns
/*void mcsEnsemble::updateEnsemblePatterns(const PatternPairMap& myPairMap)
{
   ensemblePatterns = myPairMap;
   for (MolMapITER j= ensembleMolecules.begin(); j!= ensembleMolecules.end();j++)
      {
         for(csPMapIter i = mySingMap.getMap().begin(); i!= mySingMap.getMap().end();i++)
         {
        //    ensemblePatterns.add(i->second);
            j->second.add(i->second);
         }
         cout << "Mol: " << j->second.getId()<< " contains " << j->second.moleculePatterns.getMap().size() << " patterns." << endl;
      }
}*/

void ExpaRNAEnsemble::startALL()
{
   struct timeval *Tps, *Tpf;
   struct timezone *Tzp;
   Tps = (struct timeval*) malloc(sizeof(struct timeval));
   Tpf = (struct timeval*) malloc(sizeof(struct timeval));
   Tzp = 0;

   // pattern finding
   gettimeofday (Tps, Tzp);
   EPMFinding PairMCS(ensembleMolecules.find(ensOptions.mol1Id)->second,ensembleMolecules.find(ensOptions.mol2Id)->second,ensemblePatterns,ensOptions);
   PairMCS.start();
   gettimeofday (Tpf, Tzp);
   long double time = ((Tpf->tv_sec-Tps->tv_sec)*1000000 + Tpf->tv_usec-Tps->tv_usec)/1000000.0;
   cout << "time to find EPMs in secs: " << setprecision(4) << time << endl;
   if (ensOptions.verboseOut) cout << "The EPM library contains " << ensemblePatterns.size() << " EPMs." << endl;

   //LCSEPM
   if (ensOptions.calcLCSEPM)
	   startLCSEPM();

   // start all output
   cout << endl << " -->> start writing output" << endl;

   // chdir to output
   //string dirname = "output";
   //chdir(const_cast<char*>(dirname.c_str()));

   ExpaRNAOutput outputWriter(*this);

   // postscript output
   if (ensOptions.psOut)
   {
	   cout << "    -->> Writing LCS-EPM as Postscript file...";
	   outputWriter.LCSEPMtoPS();
	   cout << "  LCS-EPM output done!"<<endl;
   }

   // output LCS-EPM as constraint input for LocARNA
   if (ensOptions.locarnaOut)
	   outputWriter.output_LocARNA();

   // output LCS-EPM as alignment
   if (ensOptions.alignOut)
	   outputWriter.output_Clustal();

   // output LCS-EPM in plain file
   if (ensOptions.epmOut)
	   outputWriter.output_LCSEPM();

   // output all EPM in plain file
   if (ensOptions.allOut)
	   outputWriter.output_allEPMs();

   // extract Alignment matchings for Coloring
   if (ensOptions.alignColor)
	   outputWriter.compareAlign();

   cout << " -->> end writing output" << endl;
}

void ExpaRNAEnsemble::startLCSEPM()
{
	struct timeval *Tps, *Tpf;
	struct timezone *Tzp;
	Tps = (struct timeval*) malloc(sizeof(struct timeval));
	Tpf = (struct timeval*) malloc(sizeof(struct timeval));
	Tzp = 0;

	gettimeofday (Tps, Tzp);
	cout << endl << " -->> starting LCS-EPM algorithm" << endl;
	if (ensOptions.verboseOut) cout << "   EPMs for LCS-EPM: " << ensemblePatterns.size() << endl;
	LCSEPM myLCSEPM(	ensembleMolecules.find(ensOptions.mol1Id)->second,
						ensembleMolecules.find(ensOptions.mol2Id)->second,
						ensemblePatterns,
						ensembleLCSEPM,
						sizeLCSEPM,
						scoreLCSEPM,
						ensOptions);

	myLCSEPM.calculateLCSEPM();

	gettimeofday (Tpf, Tzp);
	long double time = ((Tpf->tv_sec-Tps->tv_sec)*1000000 + Tpf->tv_usec-Tps->tv_usec)/1000000.0;
	cout << "   time for LCS-EPM in secs: " << setprecision(4) << time << endl;
	cout << " -->> end LCS-EPM algorithm" << endl;
}
